@@ -21,7 +21,7 @@ _all_whitespace_string = re.compile(r'\s*$')
 
 class Hist2D(object):
     """A container to hold the paramters from a 2D ROOT histogram."""
-    def __init__(self, hist, label="__nolabel__", title=None,
+    def __init__(self, hist, replace=None, label="__nolabel__", title=None,
                  xlabel=None, ylabel=None):
         try:
             if not hist.InheritsFrom("TH2"):
@@ -52,6 +52,13 @@ class Hist2D(object):
                              for i in range(nx)])
         self.y      = array([(self.yedges[i+1] + self.yedges[i])/2
                              for i in range(ny)])
+        if title: self.title = _parse_latex(title, replace)
+        else: self.title = _parse_latex(hist.GetTitle(), replace)
+        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
+        else: self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
+        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
+        else: self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
+        self.label = _parse_latex(label, replace)
     def contour(self, **kwargs):
         """Draw a contour plot."""
         _load_pyplot()
@@ -89,22 +96,33 @@ class Hist2D(object):
         sizes = self.content.flatten() / maxvalue * maxsize
         plot = plt.scatter(x, y, sizes, marker='s', **kwargs)
         return plot
-        
+
 class Hist(object):
     """A container to hold the parameters from a ROOT histogram."""
     def __init__(self, hist, replace=None, label="__nolabel__", title=None,
                  xlabel=None, ylabel=None):
         try:
-            if not hist.InheritsFrom("TH1"):
-                raise TypeError("%s does not inherit from TH1" % hist)
-        except:
-            raise TypeError("%s is not a ROOT object" % hist)
+            hist.GetNbinsX()
+            self.__init_TH1(hist)
+        except AttributeError:
+            try:
+                hist.GetN()
+                self.__init_TGraph(hist)
+            except AttributeError:
+                raise TypeError("%s is not a 1D histogram or TGraph" % hist)
+        if title: self.title = _parse_latex(title, replace)
+        else: self.title = _parse_latex(hist.GetTitle(), replace)
+        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
+        else: self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
+        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
+        else: self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
+        self.label = _parse_latex(label, replace)
+    def __init_TH1(self, hist):
         array = np.array
-        self.nbins  = n = hist.GetNbinsX()
+        self.nbins = n = hist.GetNbinsX()
         self.binlabels = _process_bin_labels([hist.GetXaxis().GetBinLabel(i)
-                                         for i in range(1, n + 1)])
-        if self.binlabels:
-            # Get rid of extra non-labeled bins
+                                              for i in range(1, n + 1)])
+        if self.binlabels: # Get rid of extra non-labeled bins
             self.nbins = n = self.binlabels.index('')
             self.binlabels = self.binlabels[:n]
         self.xedges = array([hist.GetBinLowEdge(i) for i in range(1, n + 2)])
@@ -118,31 +136,63 @@ class Hist(object):
                              for i in range(self.nbins)])
         self.underflow = hist.GetBinContent(0)
         self.overflow  = hist.GetBinContent(self.nbins + 1)
-        self.title  = _parse_latex(hist.GetTitle(), replace)
-        self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
-        self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
-        if title : self.title  = _parse_latex(title, replace)
-        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
-        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
-        self.label = _parse_latex(label, replace)
+    def __init_TGraph(self, hist):
+        array = np.array
+        self.nbins = n = hist.GetN()
+        self.x, self.y = array([]), array([])
+        x, y = ROOT.Double(0), ROOT.Double(0)
+        for i in range(n):
+            hist.GetPoint(i, x, y)
+            self.x = np.append(self.x, x)
+            self.y = np.append(self.y, y)
+        upper = array([hist.GetErrorXhigh(i) for i in xrange(n)])
+        lower = array([hist.GetErrorXlow(i)  for i in xrange(n)])
+        self.xerr = array([lower, upper])
+        upper = array([hist.GetErrorYhigh(i) for i in xrange(n)])
+        lower = array([hist.GetErrorYlow(i)  for i in xrange(n)])
+        self.yerr = array([lower, upper])
+        self.xedges = array([self.x[i] - self.xerr[0][i] for i in xrange(n)])
+        self.xedges = np.append(self.xedges,
+                                [self.x[n - 1] + self.xerr[1][n - 1]])
+        self.width = array([self.xedges[i + 1] - self.xedges[i]
+                            for i in range(n)])
+        self.underflow, self.overflow = 0, 0
+        self.binlabels = []
     def __getitem__(self, index):
-        """Return the contents of the indexth bin."""
+        """Return contents of indexth bin: x.__getitem__(y) <==> x[y]"""
         return self.y[index]
     def __setitem__(self, index, value):
-        """Set the contents of the indexth bin."""
+        """Set contents of indexth bin: x.__setitem__(i, y) <==> x[i]=y"""
         self.y[index] = value
     def __len__(self):
-        """Return the number of bins."""
+        """Return the number of bins: x.__len__() <==> len(x)"""
         return self.nbins
     def __iter__(self):
-        """Return the contents of the bins."""
+        """Iterate through bins: x.__iter__() <==> iter(x)"""
         return iter(self.y)
+    def min(self, threshold = None):
+        """Return the y-value of the bottom tip of the lowest errorbar."""
+        try:
+            lower_errors = [self.yerr[0][i] for i in range(self.nbins)]
+        except IndexError:
+            lower_errors = [self.yerr[i] for i in range(self.nbins)]
+        vals = [self.y[i] - lower_errors[i] for i in range(self.nbins)
+                if (self.y[i] - lower_errors[i]) > threshold]
+        if vals:
+            return min(vals)
+        else:
+            return threshold
     def scale(self, factor):
-        """Scale contents and errors by the given scale factor."""
+        """
+        Scale contents, errors, and over/underflow by the given scale factor.
+        """
         self.y *= factor
         self.yerr *= factor
+        self.overflow *= factor
+        self.underflow *= factor
     def show_titles(self):
         """Add the titles defined in the ROOT histogram to the figure."""
+        _load_pyplot()
         plt.title(self.title)
         plt.xlabel(self.xlabel)
         plt.ylabel(self.ylabel)
@@ -166,6 +216,7 @@ class Hist(object):
 
         All additional keyword arguments will be passed to pyplot.hist.
         """
+        kwargs.pop('fmt', None)
         _load_pyplot()
         plot = plt.hist(self.x, weights=self.y, bins=self.xedges,
                         label=self.label, **kwargs)
@@ -209,6 +260,7 @@ class Hist(object):
 
         All additional keyword arguments will be passed to pyplot.bar.
         """
+        kwargs.pop('fmt', None)
         if xerr: kwargs['xerr'] = self.xerr
         if yerr: kwargs['yerr'] = self.yerr
         _load_pyplot()
@@ -221,6 +273,7 @@ class Hist(object):
 
         All additional keyword arguments will be passed to pyplot.barh.
         """
+        kwargs.pop('fmt', None)
         if xerr: kwargs['xerr'] = self.yerr
         if yerr: kwargs['yerr'] = self.xerr
         _load_pyplot()
@@ -239,20 +292,30 @@ class Hist(object):
         for i in range(self.nbins):
             th1f.SetBinContent(i + 1, self.y[i])
         return th1f
-    def divide_wilson(self, denominator):
+    def divide(self, denominator):
         """
-        Return an efficiency plot with Wilson score interval errors.
+        Return the simple quotient with errors added in quadrature.
 
         This function is called by the division operator:
-          hist3 = hist1.divide_wilson(hist2) <--> hist3 = hist1/hist2
+            hist3 = hist1.divide_wilson(hist2) <--> hist3 = hist1 / hist2
         """
+        quotient = copy.deepcopy(self)
+        quotient.y = np.array([self[i] / denominator[i]
+                               for i in range(self.nbins)])
+        quotient.yerr = np.array([
+            quotient[i] * math.sqrt((self.yerr[i] / self[i]) ** 2 +
+                                    (denominator.yerr[i] / denominator[i])** 2)
+            for i in range(self.nbins)])
+        return quotient
+    def divide_wilson(self, denominator):
+        """Return an efficiency plot with Wilson score interval errors."""
         eff, upper_err, lower_err = wilson_interval(self.y, denominator.y)
         quotient = copy.deepcopy(self)
         quotient.y = np.array(eff)
         quotient.yerr = np.array([lower_err, upper_err])
         return quotient
     def __div__(self, denominator):
-        return self.divide_wilson(denominator)
+        return self.divide(denominator)
 
 class HistStack(object):
     """
@@ -272,23 +335,29 @@ class HistStack(object):
             for hist in hists:
                 self.add(hist, replace)
     def __getitem__(self, index):
-        """Return the indexth hist in the stack."""
+        """Return indexth hist: x.__getitem__(y) <==> x[y]"""
         return self.hists[index]
     def __setitem__(self, index, value):
-        """Replace the indexth hist in the stack with value."""
+        """Replace indexth hist with value: x.__setitem__(i, y) <==> x[i]=y"""
         self.hists[index] = value
     def __len__(self):
-        """Return the number of hists in the stack."""
+        """Return the number of hists in the stack: x.__len__() <==> len(x)"""
         return len(self.hists)
     def __iter__(self):
+        """Iterate through hists in the stack: x.__iter__() <==> iter(x)"""
         return iter(self.hists)
     def max(self):
-        """Return the value of the highest bin of all the hists in the stack."""
+        """Return the value of the highest bin of all hists in the stack."""
         maxes = [max(x) for x in self.hists]
         return max(maxes)
-    def min(self):
-        """Return the value of the lowest bin of all the hists in the stack."""
-        mins = [min(x) for x in self.hists]
+    def min(self, threshold=None):
+        """
+        Return the value of the lowest bin of all hists in the stack.
+
+        If threshold is specified, only values above the threshold will be
+        considered.
+        """
+        mins = [x.min(threshold) for x in self.hists]
         return min(mins)
     def show_titles(self):
         self.hists[-1].show_titles()
@@ -296,7 +365,7 @@ class HistStack(object):
         """
         Add a Hist object to this stack.
 
-        Any additional keyword arguments will be added to just this Hist 
+        Any additional keyword arguments will be added to just this Hist
         when the stack is plotted.
         """
         if "label" in kwargs:
@@ -343,7 +412,6 @@ class HistStack(object):
             labels.append(hist.label)
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            all_kwargs.pop('fmt', None)
             bar = ax.bar(hist.x, hist.y, zs=i, zdir='y', width=hist.width,
                          **all_kwargs)
             plots.append(bar)
@@ -365,7 +433,6 @@ class HistStack(object):
             if self.ylabel is not None: hist.ylabel = self.ylabel
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            all_kwargs.pop('fmt', None)
             bar = hist.bar(bottom=bottom, **all_kwargs)
             plots.append(bar)
             if not bottom: bottom = [0. for i in range(self.hists[0].nbins)]
@@ -387,7 +454,6 @@ class HistStack(object):
             if self.ylabel is not None: hist.xlabel = self.ylabel
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            all_kwargs.pop('fmt', None)
             bar = hist.barh(yoffset=width*i + spacer, width=width, **all_kwargs)
             plots.append(bar)
         return plots
@@ -448,12 +514,11 @@ class HistStack(object):
             if self.ylabel is not None: hist.ylabel = self.ylabel
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            all_kwargs.pop('fmt', None)
             bar = hist.bar(**all_kwargs)
             plots.append(bar)
         return plots
 
-    
+
 ################ Define functions and classes for navigating within ROOT
 
 class RootFile:
@@ -503,11 +568,24 @@ def _load_pyplot():
     import matplotlib.transforms as transforms
 
 def _parse_latex(string, replacements):
-    """Modify a string based on a dictionary of replacements."""
+    """
+    Modify a string based on a list of patterns and substitutions.
+
+    replacements should be a list of two-entry tuples, the first entry giving
+    a string to search for and the second entry giving the string with which
+    to replace it.  If replacements includes a pattern entry containing
+    'use_regexp', then all patterns will be treated as regular expressions
+    using re.sub.
+    """
     if not replacements:
         return string
-    for key, value in replacements:
-        string = string.replace(key, value)
+    if 'use_regexp' in [x for x,y in replacements]:
+        for pattern, repl in [x for x in replacements
+                              if x[0] != 'use_regexp']:
+            string = re.sub(pattern, repl, string)
+    else:
+        for pattern, repl in replacements:
+            string = string.replace(pattern, repl)
     if re.match(_all_whitespace_string, string):
         return ""
     return string
@@ -603,30 +681,30 @@ def axes_broken_y(axes, upper_frac=0.5, break_frac=0.02, ybounds=None,
         spine.set_color('none')
     return upper_axes, lower_axes
 
-def prepare_efficiency(axes, lower_bound=0.69):
-    """
-    Set up an efficiency figure with breakmarks to indicate a suppressed zero.
+## def prepare_efficiency(axes, lower_bound=0.69):
+##     """
+##     Set up an efficiency figure with breakmarks to indicate a suppressed zero.
 
-    The y-axis limits are set to (lower_bound, 1.0), as appropriate for an
-    efficiency plot, and autoscaling is turned off.
-    """
-    upper_axes, lower_axes = axes_broken_y(axes, upper_frac=0.97)
-    lower_axes.set_yticks([])
-    upper_axes.set_ylim(lower_bound, 1.)
-    upper_axes.set_autoscaley_on(False)
-    return upper_axes, lower_axes
-## ##     def breakmarks(axes, position):
-## ##         plt.axes(axes)
-## ##         xwidth = 0.005 * (plt.xlim()[1] - plt.xlim()[0])
-## ##         ywidth = 0.001
-## ##         if position == 'bottom': j = 0
-## ##         if position == 'top': j = 1
-## ##         for i in range(2):
-## ##             line = mpl.lines.Line2D(
-## ##                 [plt.xlim()[i] - xwidth, plt.xlim()[i] + xwidth],
-## ##                 [plt.ylim()[j] - ywidth, plt.ylim()[j] + ywidth],
-## ##                 linewidth=1, color='black', clip_on=False)
-## ##             axes.add_line(line)
+##     The y-axis limits are set to (lower_bound, 1.0), as appropriate for an
+##     efficiency plot, and autoscaling is turned off.
+##     """
+##     upper_axes, lower_axes = axes_broken_y(axes, upper_frac=0.97)
+##     lower_axes.set_yticks([])
+##     upper_axes.set_ylim(lower_bound, 1.)
+##     upper_axes.set_autoscaley_on(False)
+##     return upper_axes, lower_axes
+## ## ##     def breakmarks(axes, position):
+## ## ##         plt.axes(axes)
+## ## ##         xwidth = 0.005 * (plt.xlim()[1] - plt.xlim()[0])
+## ## ##         ywidth = 0.001
+## ## ##         if position == 'bottom': j = 0
+## ## ##         if position == 'top': j = 1
+## ## ##         for i in range(2):
+## ## ##             line = mpl.lines.Line2D(
+## ## ##                 [plt.xlim()[i] - xwidth, plt.xlim()[i] + xwidth],
+## ## ##                 [plt.ylim()[j] - ywidth, plt.ylim()[j] + ywidth],
+## ## ##                 linewidth=1, color='black', clip_on=False)
+## ## ##             axes.add_line(line)
 
 
 if __name__ == "__main__":
